@@ -15,25 +15,29 @@ limitations under the License.
 #ifndef TENSORFLOW_CORE_COMMON_RUNTIME_PROCESS_FUNCTION_LIBRARY_RUNTIME_H_
 #define TENSORFLOW_CORE_COMMON_RUNTIME_PROCESS_FUNCTION_LIBRARY_RUNTIME_H_
 
+#include <functional>
+#include <memory>
+#include <optional>
+#include <string>
 #include <unordered_map>
+#include <vector>
 
-// clang-format off
-// Required for IS_MOBILE_PLATFORM
-#include "tensorflow/core/platform/platform.h"
-// clang-format on
-
-#include "absl/types/optional.h"
-#include "absl/types/variant.h"
 #include "tensorflow/core/common_runtime/composite_device.h"
 #include "tensorflow/core/common_runtime/device_mgr.h"
 #include "tensorflow/core/common_runtime/device_set.h"
+#include "tensorflow/core/common_runtime/stats_publisher_interface.h"
 #include "tensorflow/core/framework/function.h"
+#include "tensorflow/core/framework/graph.pb.h"
 #include "tensorflow/core/framework/types.h"
 #include "tensorflow/core/lib/core/status.h"
+#include "tensorflow/core/platform/refcount.h"
+#include "tensorflow/core/platform/types.h"
 #include "tensorflow/core/protobuf/config.pb.h"
+#include "tsl/platform/thread_annotations.h"
+
 #if !defined(IS_MOBILE_PLATFORM)
 #include "tensorflow/core/protobuf/remote_tensor_handle.pb.h"
-#endif  // IS_MOBILE_PLATFORM
+#endif  // !IS_MOBILE_PLATFORM
 
 namespace tensorflow {
 
@@ -43,14 +47,14 @@ class FunctionArgsInterface {
 
   virtual bool HasRemoteOrPackedInputs() const = 0;
 
-  virtual Status GetLocalArg(const FunctionArgIndex& index,
-                             Tensor* val) const = 0;
+  virtual absl::Status GetLocalArg(const FunctionArgIndex& index,
+                                   Tensor* val) const = 0;
 
   virtual std::vector<Tensor> GetLocalTensors() const = 0;
 
 #if !defined(IS_MOBILE_PLATFORM)
-  virtual Status GetRemoteArg(const FunctionArgIndex& index,
-                              eager::RemoteTensorHandle* val) const {
+  virtual absl::Status GetRemoteArg(const FunctionArgIndex& index,
+                                    eager::RemoteTensorHandle* val) const {
     return errors::Unimplemented(
         "Serializing a remote argument is not implemented.");
   }
@@ -70,7 +74,8 @@ class ProcessFunctionLibraryRuntime {
       thread::ThreadPool* thread_pool = nullptr,
       DistributedFunctionLibraryRuntime* parent = nullptr,
       const SessionMetadata* session_metadata = nullptr,
-      Rendezvous::Factory rendezvous_factory = Rendezvous::Factory());
+      Rendezvous::Factory rendezvous_factory = Rendezvous::Factory(),
+      StatsPublisherFactory stats_publisher_factory = CreateNoOpStatsPublisher);
 
   ~ProcessFunctionLibraryRuntime() {
     // Deleting the FunctionLibraryRuntime map will delete the function handles
@@ -87,13 +92,12 @@ class ProcessFunctionLibraryRuntime {
   // doing the sending. `alloc_attrs` should either be empty or be the size of
   // `tensors_to_send` and indicates how the input tensors are allocated. Method
   // takes references on each of the `tensors_to_send`. Method doesn't block.
-  static Status SendTensors(const string& source_device,
-                            const string& target_device,
-                            const string& key_prefix, int64_t src_incarnation,
-                            gtl::ArraySlice<Tensor> tensors_to_send,
-                            DeviceContext* device_context,
-                            const std::vector<AllocatorAttributes>& alloc_attrs,
-                            RendezvousInterface* rendezvous);
+  static absl::Status SendTensors(
+      const string& source_device, const string& target_device,
+      const string& key_prefix, int64_t src_incarnation,
+      absl::Span<const Tensor> tensors_to_send, DeviceContext* device_context,
+      const std::vector<AllocatorAttributes>& alloc_attrs,
+      RendezvousInterface* rendezvous);
 
   // Receives `received_tensors` from `target_device` (originally sent from
   // `source_device`) using `rendezvous`. Uses `key_prefix` to construct the
@@ -114,12 +118,12 @@ class ProcessFunctionLibraryRuntime {
   FunctionLibraryRuntime* GetFLR(const string& device_name) const;
 
   // Returns the return types for the function identified by handle `h`.
-  Status GetRetTypes(FunctionLibraryRuntime::Handle h,
-                     DataTypeVector* ret_types);
+  absl::Status GetRetTypes(FunctionLibraryRuntime::Handle h,
+                           DataTypeVector* ret_types);
 
   // Returns the device incarnation for the given device_name.
-  Status GetDeviceIncarnation(const string& device_name,
-                              int64_t* incarnation) const;
+  absl::Status GetDeviceIncarnation(const string& device_name,
+                                    int64_t* incarnation) const;
 
   // For a given canonicalized key signature of the function instantiated
   // on device `device_name` and a `local_handle`, creates a handle and returns
@@ -149,37 +153,21 @@ class ProcessFunctionLibraryRuntime {
   // is set to nullptr. If some output is DT_RESOURCE, the corresponding Device*
   // is set to the device backing the resource.
   // REQUIRES: `handle` identifies a multi-device function.
-  Status GetOutputDevices(FunctionLibraryRuntime::Handle handle,
-                          std::vector<Device*>* output_devices) const;
-
-  // Returns true if function with handle `handle` was instantiated on device
-  // `device_name`. Returns false for multi-device functions.
-  bool IsInstantiatedOnDevice(const string& device_name,
-                              FunctionLibraryRuntime::Handle handle) const;
+  absl::Status GetOutputDevices(FunctionLibraryRuntime::Handle handle,
+                                std::vector<Device*>* output_devices) const;
 
   // Instantiates the function. See framework/function.h for more details.
   // Allows for function_name to be instantiated on different devices
   // as specified in attrs.
-  Status Instantiate(const string& function_name, AttrSlice attrs,
-                     const FunctionLibraryRuntime::InstantiateOptions& options,
-                     FunctionLibraryRuntime::Handle* handle);
+  absl::Status Instantiate(
+      const string& function_name, AttrSlice attrs,
+      const FunctionLibraryRuntime::InstantiateOptions& options,
+      FunctionLibraryRuntime::Handle* handle);
 
   // Returns whether the function represented by the given handle needs to
   // execute cross process.
-  Status IsCrossProcess(FunctionLibraryRuntime::Handle handle,
-                        bool* is_cross_process) const;
-
-  // TODO(iga): Reword
-  // Pins each arg that emits a `DT_RESOURCE` tensor to the device on which the
-  // corresponding resource lives. This ensures that the Placer assigns ops that
-  // access these resources to the appropriate devices.
-  static Status PinArgsAndRets(const std::vector<string>& input_devices,
-                               const std::vector<string>& output_devices,
-                               const DeviceSet& device_set,
-                               const std::vector<Node*>& arg_nodes,
-                               const std::vector<Node*>& ret_nodes,
-                               const FunctionLibraryDefinition* lib_def,
-                               Device* default_device);
+  absl::Status IsCrossProcess(FunctionLibraryRuntime::Handle handle,
+                              bool* is_cross_process) const;
 
   // Delegates to the local FLR that owns state corresponding to `handle` and
   // tells it to release it. If the `handle` isn't needed at all, the local FLR
@@ -188,12 +176,12 @@ class ProcessFunctionLibraryRuntime {
   // For multi-device functions, calls ReleaseHandle on local FLRs for each
   // component function that is part of this multi-device function.
   // Each local FLR might call RemoveHandle on this.
-  Status ReleaseHandle(FunctionLibraryRuntime::Handle handle);
+  absl::Status ReleaseHandle(FunctionLibraryRuntime::Handle handle);
 
   // Runs the function with given `handle`. Function could have been
   // instantiated on any device. More details in framework/function.h
   void Run(const FunctionLibraryRuntime::Options& opts,
-           FunctionLibraryRuntime::Handle handle, gtl::ArraySlice<Tensor> args,
+           FunctionLibraryRuntime::Handle handle, absl::Span<const Tensor> args,
            std::vector<Tensor>* rets,
            FunctionLibraryRuntime::DoneCallback done) const;
   void Run(const FunctionLibraryRuntime::Options& opts,
@@ -205,12 +193,13 @@ class ProcessFunctionLibraryRuntime {
            const FunctionArgsInterface& args, std::vector<FunctionRet>* rets,
            FunctionLibraryRuntime::DoneCallback done) const;
 
-  Status RunSync(const FunctionLibraryRuntime::Options& opts,
-                 FunctionLibraryRuntime::Handle handle,
-                 gtl::ArraySlice<Tensor> args, std::vector<Tensor>* rets) const;
-  Status RunSync(const FunctionLibraryRuntime::Options& opts,
-                 FunctionLibraryRuntime::Handle handle,
-                 CallFrameInterface* frame) const;
+  absl::Status RunSync(const FunctionLibraryRuntime::Options& opts,
+                       FunctionLibraryRuntime::Handle handle,
+                       absl::Span<const Tensor> args,
+                       std::vector<Tensor>* rets) const;
+  absl::Status RunSync(const FunctionLibraryRuntime::Options& opts,
+                       FunctionLibraryRuntime::Handle handle,
+                       CallFrameInterface* frame) const;
 
   const DeviceMgr* device_mgr() { return device_mgr_; }
 
@@ -247,11 +236,38 @@ class ProcessFunctionLibraryRuntime {
 #endif  // IS_MOBILE_PLATFORM
   };
 
+  // Structure detailing the asynchronous assumptions of a component function,
+  // such as whether it can support synchronous execution and any information
+  // needed to execute in proper order to resolve inter-subgraph dependencies.
+  class AsyncAttributes {
+   public:
+    enum Summary { kSafeForSync = 0, kSendOnly, kRecvOnly, kAsyncRequired };
+
+    AsyncAttributes()
+        : allow_control_flow_sync_execution_(false), summary_(kSafeForSync) {}
+    explicit AsyncAttributes(const Graph* graph,
+                             bool allow_control_flow_sync_execution)
+        : allow_control_flow_sync_execution_(allow_control_flow_sync_execution),
+          summary_(Summarize(graph)) {}
+    Summary summary() const { return summary_; }
+    bool allow_control_flow_sync_execution() const {
+      return allow_control_flow_sync_execution_;
+    }
+
+   private:
+    // This data member should be initialized before the summary_.
+    bool allow_control_flow_sync_execution_;
+    Summary summary_;
+    Summary Summarize(const Graph* graph);
+  };
+
   // Structure to keep track of how a component function (a single-device
   // piece of a multi-device function) fits into the multi-device function.
   struct ComponentFunctionData {
     // The handle for the instantiated component function.
     FunctionLibraryRuntime::Handle handle;
+    // The name for the component function.
+    string name;
     // arg_indices.size() is the number of arguments to the component function.
     // The i-th argument of the component function comes from the
     // `arg_indices[i]`-th argument of the multi-device function.
@@ -266,6 +282,8 @@ class ProcessFunctionLibraryRuntime {
     // ret_alloc_attrs[i] are the allocator attributes of the i-th return value
     // of the component function.
     std::vector<AllocatorAttributes> ret_alloc_attrs;
+
+    AsyncAttributes async_attributes;
   };
 
   // Data structure holding information for a single instantiated multi-device
@@ -275,12 +293,10 @@ class ProcessFunctionLibraryRuntime {
   struct MultiDeviceFunctionData {
     MultiDeviceFunctionData(const string& function_name,
                             const string& function_key, int num_outputs,
-                            FunctionLibraryDefinition&& lib_def,
                             DataTypeVector ret_types)
         : function_name_(function_name),
           function_key_(function_key),
           instantiation_counter_(1),
-          lib_def_(std::move(lib_def)),
           num_outputs_(num_outputs),
           ret_types_(std::move(ret_types)),
           is_cross_process_(false),
@@ -289,9 +305,6 @@ class ProcessFunctionLibraryRuntime {
     const string function_name_;
     const string function_key_;
     uint64 instantiation_counter_;
-    // A library that contains definitions of component functions and their
-    // transitive dependencies.
-    FunctionLibraryDefinition lib_def_;
     // Stored here to resize the output tensor vector when function is run.
     const int num_outputs_;
     DataTypeVector ret_types_;
@@ -301,6 +314,9 @@ class ProcessFunctionLibraryRuntime {
     // Indicates whether this function has remote outputs.
     bool has_remote_outputs;
 
+    //  Indicates if running this function synchronously is both allowed + safe.
+    bool enable_sync_execution;
+
     // Maps the device name to the information about the component function
     // be run on this device.
     std::unordered_map<string, ComponentFunctionData> glue_;
@@ -317,23 +333,6 @@ class ProcessFunctionLibraryRuntime {
   MultiDeviceFunctionData* IsMultiDevice(
       FunctionLibraryRuntime::Handle handle) const;
 
-  void RunMultiDevice(
-      const FunctionLibraryRuntime::Options& opts,
-      FunctionLibraryRuntime::Handle handle, std::vector<FunctionRet>* rets,
-      std::vector<std::unique_ptr<CleanUpItem>>* cleanup_items,
-      FunctionLibraryRuntime::DoneCallback done,
-      std::function<Status(const ComponentFunctionData& comp_data,
-                           InternalArgs* args)>
-          get_component_args) const;
-
-  Status CreateRendezvous(const FunctionLibraryRuntime::Options& opts,
-                          Rendezvous** created_rendezvous) const;
-
-  FunctionLibraryRuntime::DoneCallback ApplyCleanUpToDoneCallback(
-      std::vector<std::unique_ptr<CleanUpItem>>* items,
-      FunctionLibraryRuntime::DoneCallback done, const int64_t step_id,
-      const Rendezvous* rendezvous) const;
-
   DistributedFunctionLibraryRuntime* const parent_;
 
  private:
@@ -344,15 +343,15 @@ class ProcessFunctionLibraryRuntime {
 
   // For a given device_name, returns a DeviceContext for copying
   // tensors to/from the device.
-  Status GetDeviceContext(const string& device_name,
-                          DeviceContext** device_context) const;
+  absl::Status GetDeviceContext(const string& device_name,
+                                DeviceContext** device_context) const;
 
   // Looks up the information for the given `handle` and returns the name
   // of the device where the function is registered.
   string GetDeviceName(FunctionLibraryRuntime::Handle handle) const;
 
   // Removes handle from the state owned by this object.
-  Status RemoveHandle(FunctionLibraryRuntime::Handle handle);
+  absl::Status RemoveHandle(FunctionLibraryRuntime::Handle handle);
 
   // Clones ProcessFunctionLibraryRuntime and FunctionLibraryDefinition
   // (transferring ownership of both to the caller). Note that the
@@ -366,15 +365,15 @@ class ProcessFunctionLibraryRuntime {
   // FunctionLibraryDefinitions for its functions independently (and passes
   // these into the FunctionLibraryRuntime through an overlay), to avoid linear
   // runtime w.r.t. to number of functions in the current function library.
-  Status Clone(Env* env, int graph_def_version,
-               const OptimizerOptions& optimizer_options,
-               std::unique_ptr<FunctionLibraryDefinition>* out_lib_def,
-               std::unique_ptr<ProcessFunctionLibraryRuntime>* out_pflr,
-               bool skip_flib_def = false) const;
+  absl::Status Clone(Env* env, int graph_def_version,
+                     const OptimizerOptions& optimizer_options,
+                     std::unique_ptr<FunctionLibraryDefinition>* out_lib_def,
+                     std::unique_ptr<ProcessFunctionLibraryRuntime>* out_pflr,
+                     bool skip_flib_def = false) const;
 
-  Status ReleaseMultiDeviceHandle(FunctionLibraryRuntime::Handle handle);
+  absl::Status ReleaseMultiDeviceHandle(FunctionLibraryRuntime::Handle handle);
 
-  Status InstantiateMultiDevice(
+  absl::Status InstantiateMultiDevice(
       const string& function_name, AttrSlice attrs,
       const FunctionLibraryRuntime::InstantiateOptions& options,
       FunctionLibraryRuntime::Handle* handle);
@@ -389,16 +388,66 @@ class ProcessFunctionLibraryRuntime {
       const std::unique_ptr<MultiDeviceFunctionData> data,
       const string& function_key);
 
+  bool HasMultiDeviceHandle(FunctionLibraryRuntime::Handle handle) const;
+
   void RunInternal(const FunctionLibraryRuntime::Options& opts,
                    FunctionLibraryRuntime::Handle handle,
-                   gtl::ArraySlice<FunctionArg> args,
+                   absl::Span<const FunctionArg> args,
                    std::vector<FunctionRet>* rets,
                    std::vector<std::unique_ptr<CleanUpItem>>* cleanup_items,
                    FunctionLibraryRuntime::DoneCallback done) const;
 
+  absl::Status CreateRendezvous(
+      FunctionLibraryRuntime::Options& opts,
+      tsl::core::RefCountPtr<Rendezvous>* created_rendezvous) const;
+
+  FunctionLibraryRuntime::DoneCallback ApplyCleanUpToDoneCallback(
+      std::vector<std::unique_ptr<CleanUpItem>>* items,
+      FunctionLibraryRuntime::DoneCallback done,
+      const FunctionLibraryRuntime::Options& opts,
+      tsl::core::RefCountPtr<Rendezvous> rendezvous) const;
+
   void CleanUp(std::vector<std::unique_ptr<CleanUpItem>>* items,
                FunctionLibraryRuntime::DoneCallback done) const;
 
+  static absl::Status GetComponentArgs(absl::Span<const Tensor> args,
+                                       const ComponentFunctionData& comp_data,
+                                       InternalArgs* comp_args);
+
+#if !defined(IS_MOBILE_PLATFORM)
+  static absl::Status GetComponentArgs(const FunctionArgsInterface& args,
+                                       const ComponentFunctionData& comp_data,
+                                       InternalArgs* comp_args);
+#endif  // IS_MOBILE_PLATFORM
+
+  std::vector<string> GetOrderedSubgraphs(
+      const MultiDeviceFunctionData* data) const;
+
+  absl::Status PrepareRunMultiDevice(
+      const FunctionLibraryRuntime::Options& opts,
+      FunctionLibraryRuntime::Handle handle,
+      const MultiDeviceFunctionData** data) const;
+
+  absl::Status RunMultiDeviceSync(
+      const FunctionLibraryRuntime::Options& opts,
+      FunctionLibraryRuntime::Handle handle, std::vector<FunctionRet>* rets,
+      std::function<absl::Status(const ComponentFunctionData& comp_data,
+                                 InternalArgs* args)>
+          get_component_args) const;
+
+  void RunMultiDeviceAsync(
+      const FunctionLibraryRuntime::Options& opts,
+      FunctionLibraryRuntime::Handle handle, std::vector<FunctionRet>* rets,
+      std::vector<std::unique_ptr<CleanUpItem>>* cleanup_items,
+      FunctionLibraryRuntime::DoneCallback done,
+      std::function<absl::Status(const ComponentFunctionData& comp_data,
+                                 InternalArgs* args)>
+          get_component_args) const;
+
+  void PublishSubgraphs(
+      const std::string& function_name,
+      std::vector<core::RefCountPtr<FunctionRecord>>&& function_records);
+
   // Data structure holding information for a single instantiated remote
   // (to be executed on `target_device`) function.
   class FunctionData {
@@ -439,14 +488,14 @@ class ProcessFunctionLibraryRuntime {
     const string function_key_;
     bool is_cross_process_ TF_GUARDED_BY(mu_) = false;
     bool init_started_ TF_GUARDED_BY(mu_) = false;
-    Status init_result_ TF_GUARDED_BY(mu_);
+    absl::Status init_result_ TF_GUARDED_BY(mu_);
     Notification init_done_;
   };
 
   mutable mutex mu_;
 
   Env* const env_;
-  const absl::optional<const ConfigProto> config_;
+  const std::optional<const ConfigProto> config_;
   const DeviceMgr* const device_mgr_;
   const FunctionLibraryDefinition* lib_def_;
   thread::ThreadPool* default_thread_pool_;
@@ -475,7 +524,7 @@ class ProcessFunctionLibraryRuntime {
       mdevice_data_ TF_GUARDED_BY(mu_);
 
   std::unique_ptr<
-      std::unordered_map<Device*, std::unique_ptr<FunctionLibraryRuntime>>>
+      std::unordered_map<Device*, core::RefCountPtr<FunctionLibraryRuntime>>>
       flr_map_;
   int next_handle_ TF_GUARDED_BY(mu_);
   const SessionMetadata* const session_metadata_;
@@ -483,6 +532,12 @@ class ProcessFunctionLibraryRuntime {
 
   const OptimizerOptions optimizer_options_;
   const int graph_def_version_;
+
+  StatsPublisherFactory stats_publisher_factory_;
+  // Holds all stats publishers, one for publishing subgraphs of each
+  // instantiated function.
+  std::vector<std::unique_ptr<StatsPublisherInterface>> stats_publishers_
+      TF_GUARDED_BY(mu_);
 };
 
 }  // namespace tensorflow
