@@ -17,12 +17,13 @@ limitations under the License.
 
 #include <memory>
 
-#include "tensorflow/core/profiler/profiler_service.grpc.pb.h"
+#include "xla/tsl/profiler/rpc/profiler_service_impl.h"
+#include "tsl/profiler/protobuf/profiler_service.grpc.pb.h"
 
 namespace tensorflow {
 namespace profiler {
 
-std::unique_ptr<grpc::ProfilerService::Service> CreateProfilerService();
+using tsl::profiler::CreateProfilerService;  // NOLINT
 
 }  // namespace profiler
 }  // namespace tensorflow
